#!/usr/bin/perl
use strict;

# Define variables
my @daynames=('Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday');
my @monthnames=('January','February','March','April','May','June','July','August','September','October','November','December');
my @monthlengths=(31,28,31,30,31,30,31,31,30,31,30,31);
my $julian=0;

sub td2
{ #time delta
my@m=(0,31,59,90,120,151,181,212,243,273,304,334);
my($a,$b)=map{int(($_[2+$_]-1)*365.25)+$m[$_[0+$_]-1]+$_[1+$_]}(0,3);
return$b-$a;
}

sub ml
{
  # Returns month length given year and month
  my ($year,$month)=@_;
  if($month<1||$month>12){return 0;}
  if($month!=2){return $monthlengths[$month-1];}
  if($year%4==0){return 29;}   # Good until 2100
  return 28;
}

my $timezone=(localtime)[2]-(gmtime)[2]; # quick hack. Will do more timezone work later.
sub timearray
{
  my $ds=shift;  # datestring
  my ($yr,$mo,$dy,$hh,$mm);
  if($ds=~/(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)\d\d(Z?)/)
  {
    $yr=$1;$mo=$2;$dy=$3;$hh=$4;$mm=$5;
    if($6){$hh+=$timezone;} # Change timezone to local if it was GMT
    if($hh>24)
    {
      $hh-=24;
      $dy++;
      if($dy>ml($yr,$mo))
      {
        $dy=1;
        $mo++;
        if($mo>12){$mo=1;$yr++;}
      }
    }
    if($hh<0)
    {
      $hh+=24;
      $dy--;
      if($dy<1)
      {
        $mo--;
        if($mo<1){$mo=12;$yr--;}
        $dy=ml($yr,$mo);
      }
    }
    return ($yr,$mo,$dy,$hh,$mm);
  }
  elsif($ds=~/(\d{4})(\d\d)(\d\d)/){return ($1,$2,$3,'00','00');}
  else{return 0;}
}


sub firstday
{
# Given month and year, sets global $julian as Julian day of first day of month
# and returns day of week (0 - 6) of first day of month
my $month=(shift)-1;
my $year=shift;
my @days=(0,31,59,90,120,151,181,212,243,273,304,334);
$julian=$days[$month];
if(($month>1)&&($year%4==0)){$julian++;}
my $a=($year-1901)%28;
my $b=int($a/4);
my $j1=(2+$a+$b)%7;
my $fd=($julian+$j1)%7;
return($fd);
}

# Parse query string and cookies
my $query = $ENV{'QUERY_STRING'} || '';
my $cookies = $ENV{'HTTP_COOKIE'} || '';
my %refs=();
my %cookie=();
my($param,$value);
my @pairs = split(/[&;]/,$query);
foreach (@pairs)
	{
	($param,$value) = split('=',$_,2);
	$value = '' unless defined $value;
	$refs{$param}=$value;
	}

my $m=$refs{'month'}||(localtime)[4]+1;
my $y=$refs{'year'}||(localtime)[5]+1900;
#my $m=$refs{'month'}||12;
#my $y=$refs{'year'}||2000;

sub getrruledays
{
  my ($rule,@ds)=@_;
  my @dtend;
  my @rulearray=split /\s+/,$rule;
  my @days=();
  my $td=td2($m,'01',$y,$ds[1],$ds[2],$ds[0]);
  my $monthday1=firstday($m,$y);
  my $inc;  # increment count (in days)
  my $cnt;  # number of repetitions
  my @dayarray;
  my %weekdays=('SU',1,'MO',2,'TU',3,'WE',4,'TH',5,'FR',6,'SA',7);
  my $ml=ml($y,$m);
  my $offset;

  for ($rulearray[0])
  {
    if (/^D(\d+)/) # Daily recurrence
      {
      #Daily rule with $1 as increment
      $inc=$1;
      if($rulearray[1]=~/^#(\d+)/){$cnt=$1;}
      else
      {
        @dtend=timearray($rulearray[1]);
        $cnt=td2($ds[1],$ds[2],$ds[0],$dtend[1],$dtend[2],$dtend[0]);
        $cnt=int($cnt/$inc)+1;
      }
      $td++;
      for(1..$cnt)
      {
        if($td>0&&$td<=$ml){push @days,$td;}
        $td+=$inc;
      }
    }
    elsif (/^W(\d+)/) # Weekly recurrence
    {
      $inc=$1;
      for(1..$#rulearray)
      {
        for($rulearray[$_])
        {
          if($weekdays{$_}){push @dayarray,$weekdays{$_};}
          elsif(/^#(\d+)/){$cnt=$1;}
          elsif(/^\d{8}/)
          {
            if ($_<$y.$m.'01'){return;}
            @dtend=timearray($_);
	    if($dtend[0].$dtend[1] eq $y.$m){$ml=$dtend[2];}
          }
        }
      }
      if(!$cnt)
      {
        $td=0;
        $cnt=5;
      }
      elsif($td+($cnt*$inc*7)<0){return;}
      $offset=($td+$monthday1)%7;
      if($td<0){$offset=7-$offset;}
      my $tmpcnt;
      for(1..$cnt)
      {
	for(@dayarray){if($_>0)
	{
          $tmpcnt=$td+$offset+$_;
	  if($tmpcnt>0&&$tmpcnt<=$ml){push @days,$tmpcnt;}
	}}
	$td+=($inc*7);
      }
    }
  }
  return (@days);
}

# Open calendar file and create hash of
# days of month and events
my $monthstart=firstday($m,$y);

my $CAL;
my $line;
my %eventhash=();
my $numevents=0;
my $state='open';
my $last='';
my @files =</var/www/calendar/*.vcs>;
#my @files =<*.vcs>;

for(@files)
{
  $state='';
  open $CAL,"$_";
  while (<$CAL>)
  {
    # Assumes legal vcalendar stream
    # Events not properly nested between Vcalendar and Vevent
    # envelopes will not be processed
    $line=$_;
    if($state eq 'calendar')
    {
      if($line=~/^END\:VCALENDAR/){$state='open';}
      elsif($line=~/^BEGIN\:VEVENT/){$state='event';}
    }
    elsif($state eq 'event')
    {
      if($line=~/^END\:VEVENT/){$state='calendar';$numevents++;}
      elsif($line=~/^BEGIN\:VALARM/){$state='alarm';}
      elsif($line=~/^\s+(.*?)\r?\n/){$eventhash{$numevents}{$last}.=$1;}
      elsif($line=~/^(\w+)\:?(.*?)\r?\n/){$eventhash{$numevents}{$1}.=$2;$last=$1;}
    }
    elsif($state eq 'alarm')
    {
      #---to do---
      # integrate alarms with atd jobs to send emails
      # make part of separate module run when new .vcs
      # files are uploaded
      #---to do---
      if($line=~/^END\:VALARM/){$state='event';}
    }
    elsif($line=~/^BEGIN\:VCALENDAR/){$state='calendar';}
  }
  close $CAL;
}


my $curhash;
my $curhashkey;
my @tmparray;
my $tmpday;
my $dtstart;
my %dayshash=();

for(keys %eventhash)
{
  $curhashkey=$_;
  $curhash=$eventhash{$curhashkey};
  # check for possibles
  @tmparray=timearray($$curhash{DTSTART});
  $dtstart=$tmparray[0].$tmparray[1];
  if($dtstart<=$y.$m)
  {
    # Possible match
    if($$curhash{RRULE})
    {
      # Repeating event
      # Determine all days in month that the event will occur on,
      # then add hash value $dayshash{(day)}{$curhashkey}=1 for each day
      my @rruledays=getrruledays($$curhash{RRULE},@tmparray);
      for (@rruledays)
      {
        $tmpday=sprintf("%02d",$_);
        $dayshash{$tmpday}{$curhashkey}=1;
      }
    }
    elsif($dtstart eq $y.$m)
    {
       # One time event
       $dayshash{$tmparray[2]}{$curhashkey}=1;
    }
  }
}

sub timesort
{
  my $curhash=$eventhash{$_[0]};
  my @start=timearray($$curhash{DTSTART});
  return $start[3].$start[4];
}

sub gettoday
{
  my ($otype,$day)=@_;
  my $twodigitday=sprintf("%02d",$day);
  my $tmphash=$dayshash{$twodigitday};
  my $curhash;
  my $output='';
  my @start;
  if($otype eq 'cal')
  {
    my $summary;
    my $cnt=0;
    for(sort {timesort($a) <=> timesort($b)} keys %$tmphash)
    {
      if($cnt>1){$output.='*more</a></pre></div>';return $output;}
      if($cnt==0){$output="<div><pre><a href=\"javascript:dopopup($day)\">";}
      else{$output.="\n";}
      $curhash=$eventhash{$_};
      $summary=$$curhash{SUMMARY};
      if(length($summary)>11)
      {
        $summary=substr($summary,0,11);
        if($cnt>0){$summary.='..';}
      }
      $output.=$summary."\n";
      @start=timearray($$curhash{DTSTART});
      $output.=$start[3].':'.$start[4].' - ';
      @start=timearray($$curhash{DTEND});
      $output.=$start[3].':'.$start[4]."\n";
      $cnt++;
    }
    if($output){$output.='</a></pre></div>';}
  }
  elsif($otype eq 'js')
  {
    for(sort {timesort($a) <=> timesort($b)} keys %$tmphash)
    {
      $curhash=$eventhash{$_};
      $output.=$$curhash{SUMMARY}.'<br>';
      if($$curhash{DESCRIPTION}){$output.=$$curhash{DESCRIPTION}.'<br>';}
      @start=timearray($$curhash{DTSTART});
      $output.=$start[3].':'.$start[4].' - ';
      @start=timearray($$curhash{DTEND});
      $output.=$start[3].':'.$start[4].'<br><br>';
    }
  }
  return $output;
}



# Output web page for month $m and year $y
# defaults to current month and year
print "Content-type: text/html\n\n";
print<<EOF;
<html><head>
<title>Calendar</title>
<style>
body {background:black;color:yellow;font-size:12px;text-align:center;}
#event {position:absolute;background:gray;width:400px;top:75px;z-index:1;left:50%;margin-left:-200px;display:none;text-align:left;padding-left:10px;}
.calbody {position:relative;width:709px;height:102px;text-align:left;background:green;clear:left;margin:0 auto;}
.calcell {position:relative;float:left;height:100px;width:100px;background:black;margin-top:1px;margin-left:1px;overflow:hidden;}
.tc {position:absolute;width:100px;top:0px;text-align:center;}
.tr {position:absolute;width:98px;text-align:right;font-size:16px;top:1px;}
.br {position:absolute;width:98px;font-size:10px;top:88px;}
pre {font-size:12px;overflow:hidden;line-height:13px;top:10px;position:relative;}
a {color:#C0FFFF;text-decoration:none;}
.month {font-weight:bold;font-size:20px;width:100%;margin:0 auto;text-align:center;height:30px;}
a.month {color:yellow;text-decoration:underline;}
</style>
<script>
day=new Array();
EOF

my $daystring;
for(1..ml($y,$m))
{
$daystring=gettoday('js',$_);
if ($daystring){print "day[$_]=\"$daystring\";\n";}
}
print 'day[32]="Choose new month\n<form method=get action=/cgi-bin/calendar.cgi>\n<select name=month>\n';
for (1..12)
{
  print "<option value=$_";
  if($_ eq $m){print ' SELECTED';}
  print ">$monthnames[$_-1]\\n";
}
print '</select>\n<select name=year>\n';
for ($y-5..$y+5)
{
  print "<option value=$_";
  if($_ eq $y){print ' SELECTED';}
  print ">$_\\n";
}
print '</select>\n<input type=submit value=\"Get month\">\n</form>\n";';
print<<EOF;

function dopopup(n)
{
  var outstring='<span style="font-size:20px;">';
  outstring+=day[n];
  outstring+='</span><br><br><a href="javascript:dohide()">Close</a>';
  document.getElementById('event').innerHTML=outstring;
  document.getElementById('event').style.display='block';
}
function dohide()
{document.getElementById('event').style.display='none';}
</script></head>
<body><div id="event"></div>
<div class="month">
<a class="month" href="javascript:dopopup(32)">$monthnames[$m-1] $y</a></div>
EOF

my $temp=0;
my $dayofmonth=1;
my $end=ml($y,$m)+$monthstart;
my $remainder=$end%7;
if($remainder>0){$end+=7-$remainder;}
while($temp<$end)
	{
    if(($temp%7)==0){print "<div class=\"calbody\">";}
	print "<div class=\"calcell\">\n";
	if($temp<7){print "<span class=\"tc\">$daynames[$temp]</span>\n";}
	$temp++;
	if($temp>$monthstart&&$dayofmonth<=ml($y,$m))
		{
		print "<div class=\"tr\">$dayofmonth</div>\n";
		print gettoday('cal',$dayofmonth);
		$dayofmonth++;
		$julian++;
		print "<span class=\"br\">$julian</span>\n";
		}
	print "</div>\n";
    if((($temp%7)==0)||$temp==$end){print "</div>";}
	}
print "</body></html>\n";
