#!/usr/bin/perl
use strict;
use CPAN;
use Cwd;

#    Copyright 2002 Curtis Autery
#
#    This file is part of atengine.
#
#    atengine is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    atengine is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with atengine; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# Note : atengine requires IO::Socket and URI::Escape
# This script will try to auto-install them using the CPAN installer.
# If you would rather handle this manually, change the $autoinstall
# variable to 0 before running this script.

sub getdir
{
  my $valid=0;
  my $answer;
  my ($type,$default,$virtual)=@_;
  my $blurb='Directory to install ';
  if ($virtual){$blurb='Alias directory for ';}
  if ($virtual==2){$blurb='';}
  while(!$valid)
  {
    print "$blurb$type [$default]\n : ";
    chomp($answer=<STDIN>);
    $answer=$answer || $default;
    $answer=~s/^(.*?)\/?$/$1/;
    if($virtual || chdir $answer){$valid=1;}
    else{print "Can't access directory \"$answer\". Try again\n";}
  }
  return $answer;
}

sub docopy
{
    my ($sourcedir,$destdir,$filename,$dr,$dra,$sa,$pt,$sn)=@_;
    my $sourcefile=$sourcedir.'/'.$filename;
    my $destfile=$destdir.'/'.$filename;
    print "Copying $filename\n";
    die("Can't open $sourcefile") unless open(INFILE,$sourcefile);
    die ("Can't create $destfile") unless open(OUTFILE,'>'.$destfile);
    if($filename=~/(bmp|dat|jpg|htaccess)$/)
    {
	binmode INFILE;
	binmode OUTFILE;
	$/=\8192;
        while(<INFILE>){print OUTFILE $_;}
    }
    else
    {
	$/="\n";
        while(<INFILE>)
        {
            s#/var/www#$dr#g;
            s#/cgi\-bin#$sa#g;
            s#\'/atengine#\'$dra#g;
            s/attitle/$pt/g;
            s/\$ENV\{\'SERVER_NAME\'\}/\"$sn\"/g;
	    print OUTFILE $_;
        }
    }
    close INFILE;
    close OUTFILE;
    if($filename=~/(cgi|pl)$/){chmod 0755,"$destfile";}
    else{chmod 0644, "$destfile";}
}

my $autoinstall=1;
my @required_modules = qw(IO::Socket URI::Escape);
my $instdir=getcwd();
foreach my $module ( @required_modules )
{
    print "Checking for $module - ";
    eval "require $module";
    if($@)
    {
	print "not installed\n";
	if($autoinstall)
        {
	    print "Installing $module\n";
	    CPAN::Shell->install($module);
	}
        else
        {
	    print "Aborting\n";
            exit;
        }

    }
    else {print "installed\n";}
}

my $docroot=getdir('HTML files','/var/www');
my $scripts=getdir('CGI scripts','/usr/lib/cgi-bin');
my $docrootalias=getdir('HTML files','/',1);
my $scriptalias=getdir('CGI scripts','/cgi-bin',1);
my $pt=getdir('Page title','atengine',2);
my $servername=getdir('Server name (default lets web server decide)',"\$ENV{'SERVER_NAME'}",2);
my @dirs=($docroot,$docrootalias,$scriptalias,$pt,$servername);

docopy($instdir.'/home',$docroot,'index.cgi',@dirs); # docroot's only cgi script

# Install CGI scripts
chdir($instdir.'/cgi');
for(<*.cgi>){docopy($instdir.'/cgi',$scripts,$_,@dirs);}

# Create subdirectorites
my @subdirs=('css','home','user','journal','photo','photo/sample','img','img/verdana','img/digital','calendar');
for(@subdirs)
{
    print "Creating subdirectory $_\n";
    if(chdir $docroot.'/'.$_){print "already exists\n";}
    else{die("Can't create subdirectory $_") unless(mkdir $docroot.'/'.$_, 0755);}
}

# Install CSS files
my @cssfiles=('yahoo','cq','standard','nausicaa');
for(@cssfiles){docopy($instdir.'/css',$docroot.'/css',$_.'.css',@dirs);}

# Install home files
my @homefiles=('01.blg','02.blg','actionstace.jpg','customize.txt','index.txt');
for(@homefiles){docopy($instdir.'/home',$docroot.'/home',$_,@dirs);}

# Install journal files
my @journalfiles=('compose.html','decode.html','faq.txt','index.txt');
for(@journalfiles){docopy($instdir.'/journal',$docroot.'/journal',$_,@dirs);}

# Install user files
docopy($instdir.'/user',$docroot.'/user','newuser.txt',@dirs);
docopy($instdir.'/user',$docroot.'/user','.htaccess',@dirs);

# Install photo album files
docopy($instdir.'/photo',$docroot.'/photo','makealbum.pl',@dirs);
my @photofiles=('photo.dat','at.jpg','sunset.jpg');
for(@photofiles){docopy($instdir.'/photo',$docroot.'/photo/sample',$_,@dirs);}
chdir($docroot.'/photo');
`./makealbum.pl>samplealbum.html`;

# Install counter files
chdir($instdir.'/counter');
for(<*/*.bmp>){docopy($instdir.'/counter',$docroot.'/img',$_,@dirs);}

# Install calendar files
docopy($instdir.'/calendar',$docroot.'/calendar','onetime.vcs',@dirs);
docopy($instdir.'/calendar',$docroot.'/calendar','weekly.vcs',@dirs);

print "\nInstallation successful.\n";
print "Counter and Journal will not work until web server\n";
print "has write access to $docroot/user.\n\n";
print "Open $scriptalias/at.cgi to see sample page.\n";
print "Read $instdir/INSTALL and $instdir/README for\n";
print "instructions on using atengine.\n\n";

